// -*- mode: ObjC -*-
#ifndef _INCLUDED_AGSBACKEND_H
#define _INCLUDED_AGSBACKEND_H
/**
   <title>AGSBackend, a super class for outputing in different formats</title>
   Copyright (C) 2003, Willem Rein Oudshoorn.

   Written by: Willem Rein Oudshoorn <woudshoo@xs4all.nl>
   Created: February 2003

   This file will be part of the GNUstep Project
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   You should have received a copy of the GNU General Public
   License along with this program; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <Foundation/GSXML.h>
#include "AGSBackendProtocol.h"

/**
 * This class is designed to be a superclass for
 * the actual output classes.
 *
 *
 */  
@interface AGSBackend: NSObject <AGSBackendProtocol>
{
    NSMutableString *buffer;
    
    int       entityType;
    NSDictionary *entityMapping;

    AGSIndex       *globalRefs;
    AGSIndex       *localRefs;
    AGSIndex       *projectRefs;
    
    NSMutableArray *prefixStack;
    NSMutableArray *postfixStack;
    NSMutableArray *nameMap;
    
}

/**
 * Method that will parse the gsdoc fileName and initialize
 * the local references.
 *
 * It will return the rootnode of the parsed document.  If an error
 * occurs it will return nil, after displaying a warning.
 */
- (GSXMLNode*) rootNodeAfterSetupFromFile: (NSString*) fileName;

/**
 * Method that will return the result of converting the node to
 * TeX.  The result should be a valid TeX document assuming <code>tex</code>
 * can find the <code>gsdocmacros.tex</code> file.
 *
 * Normally you first want to set the references before invoking this method.
 *
 */
- (NSString*) outputDocument: (GSXMLNode*) node;

/**
 * This method will return the content of a central index file.
 * The content is created using the references set earlier.
 *
 * The method in this class does not nothing.  Should override
 *
 */
- (NSString*) tableOfContents;

/**
 * process node and append the result to the text build
 * in <code>buffer</code>.  
 */
- (void) outputNode: (GSXMLNode*) node;
/**
 * process <code>node</code> and all subsequent nodes by using
 * <code>outputNode:</code>
 */
- (void) outputNodeList: (GSXMLNode*) node;
/**
 * The dictionary content should map a
 * key (of type <code>NSString</code>) to an array of
 * <code>GSXMLNode</code>'s.
 *
 * The nodes outputed are the nodes in the array that results
 * from looking up key in content.
 *
 * The content dictionary has the key key
 * removed.   This is ok because normally you do not want to output the
 * nodes twice.  And it gives you the oppertunity to output the resulting
 * dictionary with <code>outputNodesOf:</code> without duplicating nodes.
 */
- (void) outputNodesOf: (NSMutableDictionary*) content forKeyAndRemoveKey: (NSString*) key;
/**
 * This method has the same assumption on the content dictionary
 * as the method <code>outputNodesOf:forKeyAndRemoveKey:</code>.
 *
 * This will output all nodes present in <code>content</code>
 */
- (void) outputNodesOf: (NSDictionary*) content;
/**
 * This will return an NSMutableDictionary in the form expected by
 * <code>outputNodesOf:forKeyAndRemoveKey:</code> and <code>outputNodesOf:</code>
 */
- (NSMutableDictionary*) collectChildNodesByName: (GSXMLNode*) node;
/**
 * Will iterate over the array and output every node by calling
 * <code>outputNode:</code>
 */
- (void) outputNodeArray: (NSArray*) array;

/**
   Input as an array of GSXMLNodes that are class or instance methods.

   This method will call the method [AGSBackend-outputClassMethods:] and
   [AGSBackend-outputInstanceMethods:] with as argument an array
   containing the class cq instance methods.
*/
- (void) outputClassesAndInstanceMethodsFromArray: (NSArray*) methods;

/**
 * Add a new prefix/postfix context to the stack.  All entries that
 * are added to the prefix/postfix mapping will be added to the new
 * context.  The context is removed with <code>removeLayerFromStack</code>
 */
- (void) addLayerToStack;

/**
 * Remove the toplevel prefix/postfix context from the stack.  The content
 * of this context is lost.
 */
- (void) removeLayerFromStack;

/**
 * This will add an prefix/postfix mapping to the context stack.
 *
 * Result is that an entity with name <code>name</code> will
 * result in output <code>prefix</code> <em>output of all children of entity</em>
 * followed by <code>postfix</code>.
 */
- (void) setPrefix: (NSString*) prefix forEntity: (NSString*) entity;
- (void) setPostfix: (NSString*) postfix forEntity: (NSString*) entity;
- (void) setPrefix: (NSString*) prefix andPostfix: (NSString*) postfix forEntity: (NSString*) entity;
- (NSString*) prefixForEntity: (NSString*) entity;
- (NSString*) postFixForEntity: (NSString*) entity;

/**
 * This will create a map entity in the current context stack.
 * The result is that an entity with name <code>oldName</code>
 * is treated as if it has name <code>newName</code>
 */ 
- (void) mapName: (NSString*) oldName toName: (NSString*) newName;

/**
 */
- (GSXMLNode*) _nodeOfType: (NSString*) type defining: (NSString*) what 
              descendendOf: (GSXMLNode*) node;


@end
#endif
