/**
   <title>AGSBackend, a super class for outputing in different formats</title>
   Copyright (C) 2003, Willem Rein Oudshoorn.

   Written by: Willem Reind Oudshoorn <woudshoo@xs4all.nl>
   Created: February 2003

   This file will be part of the GNUstep Project
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   You should have received a copy of the GNU General Public
   License along with this program; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <Foundation/Foundation.h>
#include "AGSBackend.h"


@interface AGSBackend (Private)

- (void) _outputUnknownNode: (GSXMLNode*) node;
- (void) _outputPrefixPostfixNode: (GSXMLNode*) node;

- (void) outputInstanceMethods: (NSArray*) methods;
- (void) outputClassMethods: (NSArray*) methods;

@end

@implementation AGSBackend

- init
{
    self = [super init];

    entityType = [GSXMLNode typeFromDescription: @"XML_ENTITY_REF_NODE"];
    entityMapping = nil;
    prefixStack = [[NSMutableArray alloc] init];
    postfixStack = [[NSMutableArray alloc] init];
    nameMap = [[NSMutableArray alloc] init];

    [self addLayerToStack];
    
    return self;
}


- (void) dealloc
{
    [entityMapping release];
    [prefixStack release];
    [postfixStack release];
    [nameMap release];
    
    [super dealloc];
}


- (void) setGlobalRefs: (AGSIndex*) gRefs
{
    ASSIGN (globalRefs, gRefs);
}

- (void) setLocalRefs: (AGSIndex*) lRefs
{
    ASSIGN (localRefs, lRefs);
}

- (void) setProjectRefs: (AGSIndex*) pRefs
{
    ASSIGN (projectRefs, pRefs);
}


- (void) addLayerToStack
{
    [prefixStack addObject: [NSMutableDictionary dictionary]];
    [postfixStack addObject: [NSMutableDictionary dictionary]];
    [nameMap addObject: [NSMutableDictionary dictionary]];
}

- (void) removeLayerFromStack
{
    [prefixStack removeLastObject];
    [postfixStack removeLastObject];
    [nameMap removeLastObject];
}


- (void) setPrefix: (NSString*) prefix forEntity: (NSString*) entity
{
    NSMutableDictionary *prefixDict = [prefixStack lastObject];

    [prefixDict setObject: prefix forKey: entity];
}

- (void) setPostfix: (NSString*) postfix forEntity: (NSString*) entity
{
    NSMutableDictionary *postfixDict = [postfixStack lastObject];

    [postfixDict setObject: postfix forKey: entity];
}

- (void) setPrefix: (NSString*) prefix andPostfix: (NSString*) postfix forEntity: (NSString*) entity
{
    [self setPrefix: prefix forEntity: entity];
    [self setPostfix: postfix forEntity: entity];
}

- (void) mapName: (NSString*) oldName toName: (NSString*) newName
{
    NSMutableDictionary *nameDict = [nameMap lastObject];

    [nameDict setObject: newName forKey: oldName];
}

- (NSString*) prefixForEntity: (NSString*) entity
{
    NSString *prefix = nil;
    NSEnumerator *num = [prefixStack reverseObjectEnumerator];
    NSDictionary *dict;

    while ((dict = [num nextObject]) && prefix == nil)
        {
        prefix = [dict objectForKey: entity];
        }
    
    return prefix;
}

- (NSString*) postFixForEntity: (NSString*) entity
{
    NSString *postfix = nil;
    NSEnumerator *num = [postfixStack reverseObjectEnumerator];
    NSDictionary *dict;

    while ((dict = [num nextObject]) && postfix == nil)
        {
        postfix = [dict objectForKey: entity];
        }
    
    return postfix;
}


- (NSString*) remapName: (NSString*) name
{
    NSString *remappedName = nil;
    NSEnumerator *num = [nameMap reverseObjectEnumerator];
    NSDictionary *dict;

    while ((dict = [num nextObject]) && remappedName == nil)
        {
        remappedName = [dict objectForKey: name];
        }

    if (remappedName)
        {
        return remappedName;
        }
    else
        {
        return name;
        }
}

- (GSXMLNode*) rootNodeAfterSetupFromFile: (NSString*) filename
{
    GSXMLParser *parser = [GSXMLParser parserWithContentsOfFile: filename];
    GSXMLNode   *root;
    AGSIndex    *lRefs;
    
    
    [parser substituteEntities: NO];
    [parser doValidityChecking: YES];
    [parser keepBlanks: NO];

    if ([parser parse] == NO)
        {
        NSLog (@"WARNING %@ is not a valid document", filename);
        return nil;
        }
    
    root = [[parser document] root];
    lRefs = [AGSIndex new];

    [localRefs makeRefs: root];

    [self setLocalRefs: lRefs];

    return root;
}

- (NSString*) outputDocument: (GSXMLNode*) node
{
    NSString *tmp;
    
    buffer = (NSMutableString*) [NSMutableString string];
    
    [self outputNode: node];

    tmp = buffer;
    buffer = nil;
    
    return tmp;
}

- (NSString*) tableOfContents
{
    return nil;
}

- (void) outputClassMethods: (NSArray*) methods
{
    [self outputNodeArray: methods];
}

- (void) outputInstanceMethods: (NSArray*) methods
{
    [self outputNodeArray: methods];
}

- (void) outputClassesAndInstanceMethodsFromArray: (NSArray *) methods
{
    NSEnumerator   *methodEnumerator;
    NSMutableArray *classMethods;
    NSMutableArray *instanceMethods;
    GSXMLNode      *method;

    methodEnumerator = [methods objectEnumerator];
    
    classMethods = [NSMutableArray array];
    instanceMethods = [NSMutableArray array];

    while ((method = [methodEnumerator nextObject]))
        {
        if ([[method objectForKey: @"factory"] boolValue])
            {
            [classMethods addObject: method];
            }
        else
            {
            [instanceMethods addObject: method];
            }
        }

    [self outputClassMethods: classMethods];
    [self outputInstanceMethods: instanceMethods];
}


- (void) outputEntity: (NSString*) name
{
    NSString *replacement = [entityMapping objectForKey: name];

    if (!replacement)
        {
        [buffer appendFormat: @"%% unknown entity: %@\n", name];
        }
    else
        {
        [buffer appendString: replacement];
        }
}

- (void) outputNodeList: (GSXMLNode*) node
{
    while (node != nil)
        {
        GSXMLNode *next = [node next];

        CREATE_AUTORELEASE_POOL (arp);
        [self outputNode: node];
        RELEASE (arp);
        
        node = next;
        }
}

- (void) outputNodeArray: (NSArray*) array
{
    NSEnumerator *num = [array objectEnumerator];
    GSXMLNode *node;

    while ((node = [num nextObject]))
        {
        CREATE_AUTORELEASE_POOL (arp);
        [self outputNode: node];
        RELEASE (arp);
        }
}

- (void) outputNodesOf: (NSMutableDictionary*) content forKeyAndRemoveKey: (NSString*) key
{
    NSArray *array = [content objectForKey: key];
    [self outputNodeArray: array];
    [content removeObjectForKey: key];
}

- (void) outputNodesOf: (NSDictionary*) content
{
    NSEnumerator *num = [content keyEnumerator];
    NSString *key;

    while ((key = [num nextObject]))
        {
        [self outputNodeArray: [content objectForKey: key]];
        }
}


- (void) outputNode: (GSXMLNode*) node
{
    NSString *name;
    int       type    = [node type];

    name = [self remapName: [node name]];
    
    if (type == entityType)
        {
        [self outputEntity: name];
        }
    else
        {
        SEL sel = NSSelectorFromString ([NSString stringWithFormat: @"_output_%@:", name]);
        SEL psel = NSSelectorFromString ([NSString stringWithFormat: @"_parse_%@:", name]);
        
        if (psel != NULL && [self respondsToSelector: psel])
            {
            [self performSelector: psel withObject: node];
            }
        
        if (sel != NULL && [self respondsToSelector: sel])
            {
            [self performSelector: sel withObject: node];
            }
        else if ([self prefixForEntity: name])
            {
            [self _outputPrefixPostfixNode: node];
            }
        else
            {
            [self _outputUnknownNode: node];
            }
        }
}

- (void) _outputUnknownNode: (GSXMLNode*) node
{
    NSLog (@"WARNING: %@ encountered an unknown node: %@, %d (%@)\n",
           self,
            [node name],
            [node type],
            [node content]);
}

- (void) _outputPrefixPostfixNode: (GSXMLNode*) node
{
    NSString *postfix = [self postFixForEntity: [node name]];
    NSMutableString *oldBuffer = buffer;

    [buffer appendString: [self prefixForEntity: [node name]]];
    buffer = (NSMutableString*) [NSMutableString string];
    [self outputNodeList: [node firstChild]];
    [oldBuffer appendString: [buffer stringByTrimmingSpaces]];
    buffer = oldBuffer;
    if (postfix)
        {
        [buffer appendString: postfix];
        }
}

- (NSMutableDictionary*) collectChildNodesByName: (GSXMLNode*) node
{
    
    NSMutableDictionary *result = [NSMutableDictionary dictionary];
    GSXMLNode *childNode = [node firstChild];
    
    while (childNode != nil)
        {
        NSString *name = [childNode name];
        NSMutableArray  *content;

        content = [result objectForKey: name];

        if (content == nil)
            {
            content = [[NSMutableArray alloc] init];
            [result setObject: content forKey: name];
            [content release];
            }

        [content addObject: childNode];
        childNode = [childNode next];
        }

    return result;
}


- (GSXMLNode*) _nodeOfType: (NSString*) type
                  defining: (NSString*) what
              descendendOf: (GSXMLNode*) node
{
    if (node == nil || ([node type] == entityType)) // CHECKME FIXME
        {
        return nil;
        }
    
    if ([[node name] compare: type] == NSOrderedSame)
        {
        if ([[node objectForKey: @"name"] compare: what] == NSOrderedSame)
            {
            return node;
            }
        }
    node = [node firstChild];
    
    while (node != nil)
        {
        GSXMLNode *returnNode;

        returnNode = [self _nodeOfType: type
                           defining: what
                           descendendOf: node];

        if (returnNode != nil)
            {
            return returnNode;
            }

        node = [node next];
        }
    return nil;
}

@end
