// -*- mode: ObjC -*-

#include <Foundation/Foundation.h>

/**
   Class that handles all the details of figuring out
   how to handle all the files that autogsdoc uses.
*/
 
@interface FileManager: NSObject
{
    @private
    
    NSMutableArray *sourceFiles;   // Array of .h, .m files
    NSMutableArray *gsdocFiles;    // Array of the .gsdoc files
    NSMutableArray *htmlFiles;     // Array of the .html files
    NSString       *referenceFile; // file containing the project references

    NSString *documentationDirectory; // directory of the resulting documenation
    NSString *headerDirectory;  // directory where we look for header files
}

- initForProject: (NSString*) project;

/**
   Returns the content of the reference file.
   If the reference file is invalid, or the does not
   exist, it will return nil.
 */
- (NSDictionary*) contentsOfReferenceFile;

/**
   Save the dictionary in the reference file, thereby
   destroying the original content.

   Returns YES if successfull, NO in case of an error.
*/
- (BOOL) writeContentsOfReferenceFile: (NSDictionary*) content;

/**
   Returns the modification timestamp of the reference file.
   If the reference file does not exist it will return distantPast.
*/
- (NSDate*) timeStampOfReferenceFile;

/**
   Delete the reference file if it exists.
*/
- (void) deleteReferenceFile;
 
/**
   Returns the complete pathname for the template.
   If the template does not exist return nil.

   The complete path for a template Xxx is determined by:

   1 - Lookup up XxxTemplate in the standardUserDefaults.
   2 - adding extension .gsdoc to the result of 1.
   3 - If the result is a relative path the path is assumed
       to be the documentation path.
*/
- (NSString*) fileNameForTemplate: (NSString*) templateName;

/**
   Returns a mapping from template name --> complete path name.

   The keys in the dictionary are the elements in the array for
   which a template exist.  The values are the complete path names
   to the template.  
*/
- (NSDictionary*) templateFileNamesFor: (NSArray*) templateNames;

/**
   Returns enumerator that enumerates all the .gsdoc files for the project
   excluding the template gsdoc files.
*/
- (NSEnumerator*) gsdocFileEnumerator;

/**
   Returns enumerator that enumerates all the source files for the project.
   Returns nil if there are no source files.
*/
- (NSEnumerator*) sourceFileEnumerator;

/**

*/
- (NSEnumerator*) htmlFileEnumerator;


/**
   Returns the full path name of the html file corresponding to the
   gsdoc file given as an argument
*/
- (NSString*) htmlFileForGsdocFile: (NSString*) gsdocFile;
- (NSString*) xlpFileForGsdocFile: (NSString*) gsdocFile;
- (NSString*) texFileForGsdocFile: (NSString*) gsdocFile;
/**
 * Search file as is, if not exist and not absolute, check
 * the documentation directory.
 */
- (NSString*) findFile: (NSString*) fileName;
/**
   Returns the gsdocFile generated for a source file.  
*/
- (NSString*) gsdocFileForSourceFile: (NSString*) sourceFile;

/**
   If the source is a header and the file name is not absolute,
   modify the path by taking the header directory into account.

   Note that we probably want to get rid of this method and
   actually want to make sure that the source file enumerator
   returned by this object has already done the fixing up.

   candidate for removing
*/
- (NSString*) fixSourceFilePath: (NSString*) sourceFile;

/**
   If the filename is not an absolute path return the same
   file name but located in the documentation directory

   candidate for removing
*/
- (NSString*) fileInDocumentationDirectory: (NSString*) fileName;

/**

   candidate for removing
*/
- (NSString*) documentationDirectory;

/**
   Add new file to the list of gsdoc's to process.
   The argument is not necessary a gsdoc file.  This method
   will try to do the right thing by figuring out which gsdoc file it should use.
*/
- (void) addFileToGsdocs: (NSString*) fileName;

@end
